<?php
/**
 * A Plugin to generate "Veranstaltungsnummern" /course numbers automatically
 *
 * @package     augsburg-medienlabor
 * @subpackage  administration
 *
 * @author    Bernhard Strehl
 * @copyright (c) Authors
 * @version   $Id$
 */
/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA	02111-1307, USA.
*/

/*

	###############################################################################
            		WARNING!!
	###############################################################################
			it is possible that this plugin is not secure, i.e. sql-injections may be possible! This has not been extensively checked.
			Since it can be only accessed by root however, this did not seem to be a severe problem.
			
*/
class Veranstaltungsnummern extends AbstractStudIPAdministrationPlugin {


    function Veranstaltungsnummern() {

        parent::AbstractStudIPAdministrationPlugin();

        $this->setPluginiconname("img/number.png");

        # navigation, only for root accessible!
        global $perm;
        if($perm->have_perm("root")) {
            $navigation =& new PluginNavigation();
            $navigation->setDisplayname(_("Veranstaltungsnummern generieren"));
            $this->setNavigation($navigation);


            # top navigation
            $top_navigation =& new PluginNavigation();
            $top_navigation->setDisplayname(_("Veranstaltungsnummern generieren"));
            $this->setTopnavigation($top_navigation);
        }
    }


    function initialize() {
    }


    function getPluginname() {
        return _("Veranstaltungsnummer ");
    }


    function actionShow() {

        global $perm;
        global $sess;
        //only for root accessible
        $perm->check("root");



        require_once("lib/msg.inc.php");
        require_once("lib/functions.php");
        require_once("lib/messaging.inc.php");
        require_once("lib/classes/UserManagement.class.php");
        require_once("lib/classes/SemesterData.class.php");
//load config
        if(file_exists(dirname(__FILE__).'/veranstaltungsnummern-config.inc.php'))
            require_once('veranstaltungsnummern-config.inc.php');
        else
            throw new Exception ('File not found -  Please copy veranstaltungsnummern-config.inc.php.dist to veranstaltungsnummern-config.inc.php');


        $db = new DB_Seminar();

        //Auswahl, welche Ausgabe oder Aktion
        if(sizeof($_POST)) {  //sql-select-appendix, needed for preview and confirm
            $select_appendix=' AND ( start_time >= '.$_POST["sem_select"].' '.($_POST["include_not_ending_seminare"]?' OR duration_time < 0 )':')')
                    .(sizeof($_POST["seminar_ids"])?$select_appendix.=' and seminar_id in ("'.implode('","', $_POST["seminar_ids"]).'")':'');

            //preview of new numbers
            if($_POST["preview_numbers_x"]) {
                if(sizeof($_POST["institutes"])) {
                    $content=	$this->getNewVorlesungsnummern($_POST["institutes"],$_POST["sem_types"],$select_appendix,  false).
                            $this->getConfirmAppendix($_POST["institutes"], $_POST["sem_types"], $_POST["sem_select"], $_POST["include_not_ending_seminare"]).
                            '</form>';
                    $msg = MessageBox::info(_("Sie sehen nun die Vorschau. Drcken Sie speichern, um die Veranstaltungsnummern umzuschreiben oder Vorschau um die Vorschau zu aktualisieren."));
                }
                else {
                    $msg =	MessageBox::error(_("Sie mssen zumindest eine Einrichtung whlen"));
                    $content = $this->getSelectForm();
                }
            }
            //confirm rewrite
            elseif($_POST["confirm_numberupdate_x"]) {
                $affectednumbers =  $this->getNewVorlesungsnummern($_POST["institutes"],$_POST["sem_types"], $select_appendix, true);
                $msg = 	MessageBox::success(sprintf(_("Es wurden %s Veranstaltungsnummern von %s Einrichtungen aktualisiert "),$affectednumbers,sizeof($_POST["institutes"])));
                $content = $this->getSelectForm();
            }

        }
        //initially, the select form.
        else {
            $msg = MessageBox::info(_("Whlen Sie die Einrichtungen, Veranstaltungstypen und ab welchem Semester deren Veranstaltungsnummern verndert werden sollen."));
            $content = $this->getSelectForm();
        }




        if($msg)
            echo $msg;
        ?>
<table border="0" align="center" cellspacing="0" cellpadding="0" width="100%">
    <tr>
        <td class="blank">
        <?= $content?>
        </td>
    </tr>
</table>	

        <?
                }

    /**
     * Generates new course-numbers. If $doupdate is set to true, the new numbers will be stored
     * in the database, else it will return a preview.
     *
     * @param array $institut_ids List of institutes affected
     * @param array $semtypes list of "seminar_types" affected
     * @param string $sql_select_appendix a string to modifiy the sql-select.
     * @return String $output preview-output if $doupdate is false
     *            $affected_numbers  count(affected seminare)
     *
     * @author Bernhard Strehl
     * @version 2010-02-04
     */
    function getNewVorlesungsnummern(array $institut_ids, array $semtypes , $sql_select_appendix , $doupdate) {
        $db = DBManager::get();

        if(!$doupdate) {
            $output = "<form action=".PluginEngine::getLink($this, array(), "show")." method='POST'><br>";

            $institute_names =  $db->query('SELECT Name from Institute where institut_id in ("'.implode('","', $institut_ids).'")  ')->fetchAll(PDO::FETCH_COLUMN);
            $inst_counter = 0;

            $output.='<br><br><b>'._("Schreibweise").':</b><br> '._('NEUE_VORLESUNGSNUMMER - (ALTE_VORLESUNGSNUMMER) - Veranstaltungstitel').'<br><br>';
        }
        else
            $affected_numbers = 0;
        foreach ($institut_ids as $institut_id) {
            if(!$doupdate)
                $output.='<b>'.$institute_names[$inst_counter++].'</b> '.$this->getInfolink('institut', $institut_id).'<br>';

            //Abhandeln aller Seminare nach Lehrstuhl geordnet
            $all_seminare  = $db->query('SELECT name, seminar_id, institut_id, veranstaltungsnummer FROM  seminare where institut_id ="'.$institut_id.'" and status in   ("'.implode('","', $semtypes).'") '.$sql_select_appendix.' order by mkdate asc  ')->fetchAll(PDO::FETCH_ASSOC);
            $initialnumber = 1;
            foreach($all_seminare as  $seminar) {
                $number  = $this->getSeminarNumber($seminar["institut_id"],  $initialnumber);
                //Konnte nummer generiert werden? => berschreiben und initial erhhen.
                if($number) {
                    if(!$doupdate) {
                        //	$output.= '<br>'.$number.'&nbsp;&nbsp;-&nbsp;&nbsp;('.$seminar["veranstaltungsnummer"].')&nbsp;&nbsp;-&nbsp;&nbsp;'.$seminar["name"].' '.$this->getInfolink('seminar', $seminar["seminar_id"]);
                        $output .= '<input name="seminar_ids[]" type="checkbox" id ="'.$seminar['seminar_id'].'" value ="'.$seminar['seminar_id'].'" checked="checked" />';
                        $output.='<label for="'.$seminar['seminar_id'].'">'.$number.'&nbsp;&nbsp;-&nbsp;&nbsp;('.$seminar["veranstaltungsnummer"].')&nbsp;&nbsp;-&nbsp;&nbsp;'.$seminar["name"].' '.$this->getInfolink('seminar', $seminar["seminar_id"]);
                        $output.='</label><br>';
                    }
                    else {
                        //preparation to update vl-number
                        $vl_number_preparation = $db->prepare("update seminare set VeranstaltungsNummer = :vanumber where seminar_id = :seminar_id");
                        //execute
                        $vl_number_preparation->execute(array('seminar_id' => $seminar["seminar_id"], 'vanumber'=>$number));
                        $affected_numbers++;
                    }

                    $initialnumber ++;
                }
            }
            $output.='<br><br><br>';
        }
        if(!$doupdate)
            $output.=  makeButton('vorschau',   "input",   _("Erneute Vorschau"), 'preview_numbers');
        if($doupdate)
            return	$affected_numbers;
        else
            return $output;

        makeButton();

    }


    /**
     * Generates a coursenumber depending of the institute and a counter
     *
     * @param string $institut_id the id of the institute the course belongs to
     * @param int $initialnumber the counter
     * @return int $number generated number
     *              or false if institut does not have a kuerzel.
     *
     * @author Bernhard Strehl
     * @version 2010-02-04
     */
    function getSeminarNumber($institut_id, $initialnumber) {


        $db = DBManager::get();

        $sql_getkuerzel =
                'select content as kuerzel from Institute
      left join 
       (select range_id, content from datafields_entries where datafield_id = (select datafield_id from datafields where name="'.EINRICHTUNGSKUERZEL_BEZEICHNUNG.'")) datfields 
      on Institute.institut_id = datfields.range_id
      where Institut_id = ?  ';
        $stmt_kuerzel = $db->prepare($sql_getkuerzel);

        //Finde Vater
        $sql_getfather =
                "select studip_object_id, item_id from range_tree where item_id = (SELECT parent_id FROM  range_tree  WHERE  item_id  = ? )";


        $stmt_getfather = $db->prepare($sql_getfather);


        $kuerzel_baum = '';



        $stmt_instid = $db->prepare ('select item_id  from range_tree where studip_object_id= ?');
        $stmt_instid->execute(array($institut_id));
        $item_id =          $stmt_instid->fetchColumn();



        $num_existing_va = $db->query('select count(*) from seminare where institut_id="'.$institut_id.'"')->fetchColumn();

        $stmt_kuerzel->execute(array($institut_id));
        $kuerzel_baum = $stmt_kuerzel->fetchColumn();
        do {
            $stmt_getfather->execute(array($item_id));
            $father =   $stmt_getfather->fetch(PDO::FETCH_ASSOC);
            $item_id = $father["item_id"];

            $institut_id = $father["studip_object_id"];

            $stmt_kuerzel->execute(array($institut_id));

            $einrichtungskuerzel =   $stmt_kuerzel->fetchColumn();


            //Krzel gesetzt
            if($einrichtungskuerzel   )
                $kuerzel_baum = $einrichtungskuerzel.TREE_SEPARATOR.$kuerzel_baum;
            //Krzel nicht gesetzt obwohl es existieren sollte
            else if($father) {
                $kuerzel_baum = false;
                break;
            }

        } while ($father);

        if(!str_replace (TREE_SEPARATOR, "", $kuerzel_baum))
            return false;

        else
            return $kuerzel_baum.TREE_SEPARATOR.veranstaltungsnummern_formatLastNumber($initialnumber);
        //		return $kuerzel_baum." ".($initialnumber);
    }


    /**
     * Generates the ConfirmAppendix, a form-Appendix.It is used to confirm the preview and accept
     * the new numbers. Basically it only stores the old parameters as hidden values.
     *
     * @param array $institut_ids  user-selected institut_ids
     * @param array $sem_types  user-selected sem_types
     * @return string $form the confirm-form
     *
     * @author Bernhard Strehl
     * @version 2010-02-04
     */

    function getConfirmAppendix(array $institut_ids, array $sem_types, $start_time, $include_not_ending_seminare) {





        foreach ($institut_ids as $institut_id) {
            $form .= '<input name="institutes[]" type="hidden"
											  value ="'.$institut_id.'" />'; 
        }
        foreach ($sem_types as $type) {
            $form .= '<input name="sem_types[]" type="hidden"
											  value ="'.$type.'" />'; 
        }
        $form.='<input type="hidden" name="sem_select" value='.$start_time.' />';
        if($include_not_ending_seminare)
            $form.='<input type="hidden" name="include_not_ending_seminare" value="true" />';

        $form .=  makeButton('speichern', $mode = "input", $tooltip = _("Neue Vorlesungsnummern generieren"), 'confirm_numberupdate');


        return $form;
    }

    /**
     * Generates the SelectForm, which is initially shown to allow the admin to select
     * institutes and sem_types
     *
     *
     * @return string $form the select-form
     *
     * @author Bernhard Strehl
     * @version 2010-02-04
     */
    function getSelectForm() {

        $db = DBManager::get();
        //get all institutes
        $all_institutes =  $db->query('SELECT institut_id, name ,content as kuerzel FROM Institute , datafields_entries where institut_id = range_id and content not like ""   order by name ')->fetchAll(PDO::FETCH_ASSOC);
        $output = '';


        $form = "<form action=".PluginEngine::getLink($this, array("userlist_submit"=> 1), "show")." method='POST'><br>";

        //select institutes
        $form .= '<fieldset>  <legend>'._('Whlen Sie die Einrichtungen, fr die Vorlesungsnummern generiert werden sollen').'.</legend>';
        $form .=_('<b>Schreibweise:</b> gewhltes Einrichtungskrzel (ohne zugehrigem Baum) - Name der Einrichtung').'<br><br>';
        foreach ($all_institutes as $inst_counter => $institute) {
            $form .= str_pad($inst_counter,3,0,STR_PAD_LEFT).' <input name="institutes[]" type="checkbox" id="'.$institute['institut_id'].'"
											  value ="'.$institute['institut_id'].'" />
						  					<label for="'.$institute['institut_id'].'">'.$institute['kuerzel'].' - '.$institute['name'].'</label><br>';


        }
        $form .=  "</fieldset>";

        //select veranstaltungs-types
        global $SEM_TYPE;
        $form .= '<br><br><br><fieldset>  <legend>'._('Whlen Sie die Veranstaltungstypen, die bercksichtigt werden sollen').'.</legend>';
        foreach($SEM_TYPE as $num=> $type) {
            $form .= '<input name="sem_types[]" type="checkbox" id="'.$num.'"
											  value ="'.$num.'" checked="checked" />
						  					<label for="'.$num.'">'.$type["name"].'</label><br>';
        }
        $form .=  "</fieldset>";

        $semdata = new SemesterData();

        $sem_select = $semdata->GetSemesterSelector(null,  1000, 'beginn', false);

        $form .= '<br><br><br><fieldset>  <legend>'._('AB welchem Semester sollen die Vorlesungsnummern generiert werden?').'</legend>';
        $form.=$sem_select;
        $form.='<br><input type="checkbox" id="include_not_ending_seminare" name="include_not_ending_seminare" value="true" checked="checked" /><label for="include_not_ending_semester">'._('Semesterbergreifende Veranstaltungen bercksichtigen').'</label><br>';
        $form .=  "</fieldset>";


        $form .=  makeButton('vorschau', $mode = "input", $tooltip = _("Vorschau der neuen Nummern"), 'preview_numbers');

        $form .=  "</form>";
        return $form;
    }

    /**
     * Returns a link to an institut / seminar
     *
     * @param string value institut | seminar
     * @param string id the id of the institut or seminar
     * @return string $link the link
     *
     * @author Bernhard Strehl
     * @version 2010-02-04
     */
    function getInfolink($whereto, $id) {
        global $ASSETS_URL;
        if($whereto == "institut")
            $url = URLHelper::getLink('institut_main.php?auswahl='.$id);
        elseif($whereto=="seminar")
            $url = URLHelper::getLink('details.php?sem_id='.$id);

        return '<a href='.$url.'><img src="'.$ASSETS_URL.'/images/link_extern.gif" alt="Zum '.$whereto.' springen"></a>';
    }


}
